"use strict";
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Config = void 0;
var fs_1 = require("fs");
var path_1 = require("path");
var deepmerge = require("deepmerge");
var scripts_1 = require("./scripts");
var path = require("path");
var Config = /** @class */ (function () {
    function Config(config) {
        this.scripts = new scripts_1.Scripts(config.scripts);
        this.menu = config.menu;
        this.options = config.options;
        this.settings = config.settings;
    }
    Object.defineProperty(Config.prototype, "customFile", {
        get: function () {
            var match = this.options.files.reverse().find(function (item) { return (0, path_1.basename)(item) === 'launcher-custom.json'; });
            if (match)
                return match;
            return 'launcher-custom.json';
        },
        enumerable: false,
        configurable: true
    });
    Config.load = function (directory) {
        var e_1, _a;
        var hash = new Set();
        var config = new Config(Config.default);
        var files = Config.default.options.files;
        var loaded;
        var loadedFiles = [];
        if (!(0, fs_1.existsSync)(directory))
            throw Error('Directory "' + directory + '" not found.');
        do {
            loaded = 0;
            try {
                for (var files_1 = (e_1 = void 0, __values(files)), files_1_1 = files_1.next(); !files_1_1.done; files_1_1 = files_1.next()) {
                    var file = files_1_1.value;
                    if (file && !hash.has(file)) {
                        var fullPath = path.join(directory, file);
                        try {
                            if ((0, fs_1.existsSync)((0, path_1.resolve)(fullPath))) {
                                config = config.merge(fullPath);
                                loadedFiles.push(fullPath);
                            }
                        }
                        catch (error) {
                            throw new Error('Error loading config file "' + fullPath + '" ' + error);
                        }
                        hash.add(file);
                        loaded++;
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (files_1_1 && !files_1_1.done && (_a = files_1.return)) _a.call(files_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
            files = config.options.files;
        } while (loaded > 0);
        return {
            files: loadedFiles,
            config: config
        };
    };
    Config.evaluateShellOption = function (shellOption, defaultOption) {
        if (typeof shellOption !== 'object')
            return shellOption;
        var shell = shellOption[process.platform];
        if (shell !== undefined)
            return shell;
        shell = shellOption.default;
        if (shell !== undefined)
            return shell;
        return defaultOption;
    };
    Config.prototype.merge = function (file) {
        var absolutePath = (0, path_1.resolve)(file);
        var current = {
            menu: this.menu,
            options: this.options,
            scripts: this.scripts.scripts,
            settings: this.settings
        };
        var config = deepmerge(current, require(absolutePath));
        Config.verifyScriptNames(config.scripts);
        return new Config(config);
    };
    Config.verifyScriptNames = function (scripts) {
        var e_2, _a;
        var hash = new Set();
        try {
            for (var _b = __values(Object.keys(scripts)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var key = _c.value;
                var value = key.replace(/\$\w+(\:|$)/g, '$1');
                if (hash.has(value))
                    throw new Error("Duplicate object key: '" + key + "'");
                hash.add(value);
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
    };
    Config.loadConfig = function (file) {
        var absolutePath = (0, path_1.resolve)(file);
        if ((0, fs_1.existsSync)(absolutePath)) {
            return require(absolutePath);
        }
        return {};
    };
    Config.default = {
        scripts: {},
        menu: {
            description: ''
        },
        options: {
            logLevel: 0,
            limit: 0,
            files: ['launcher-config.json', 'launcher-scripts.json', 'launcher-settings.json', 'launcher-menu.json', 'launcher-custom.json'],
            script: {
                shell: true
            },
            glob: {},
            menu: {
                defaultChoice: '',
                defaultScript: '',
                defaultSelect: '',
                confirm: true,
                timeout: 0,
                pageSize: 7
            }
        },
        settings: {}
    };
    return Config;
}());
exports.Config = Config;
